/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import it.eng.console.process.bo.ProcessBO;
import it.eng.console.process.bo.RelevantDataBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

import org.spagic.metadb.model.Process;

public class RelevantDataProcessAndVersionAction extends AbstractAction {
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
		throws Exception {
		String processIdFromRequest = (String)serviceRequest.getAttribute(Constants.LBL_PROCESS_ID);
		long processId = Long.parseLong(processIdFromRequest);
	
		Process process = ProcessBO.getProcessDetail(processId);
		if(processId != Constants.ALL_PROCESS){
			serviceResponse.setAttribute(Constants.LBL_VERSION, process.getProcessVersion());
			serviceResponse.setAttribute(Constants.LBL_TYPE, process.getTypeService().getValue());
			//process attributes
			SourceBean attributesBean = createProcessAttributeBean(processId);
			serviceResponse.setAttribute(attributesBean);

		}else {
			serviceResponse.setAttribute(Constants.LBL_VERSION, "");
			serviceResponse.setAttribute(Constants.LBL_TYPE, Constants.ANY_TYPE);
		}
	}
	
	protected SourceBean createProcessAttributeBean(long processId) {
		SourceBean attributesBean = null;
		try {
			attributesBean = new SourceBean("PROCESS_ATTRIBUTES");
			List<List> attributesList = RelevantDataBO.getAttributeRelevant(processId);
			SourceBean attributeBean = null;
			String catalog = null;
			for (List attribute : attributesList) {
				attributeBean = new SourceBean("PROCESS_ATTRIBUTE");
				attributeBean.setAttribute(Constants.LBL_ID_ATTRIBUTE, attribute.get(0));
				attributeBean.setAttribute(Constants.LBL_NAME, attribute.get(1));
				catalog = (attribute.get(2) != null)
							? (String)attribute.get(2)
							: "";
				attributeBean.setAttribute(Constants.LBL_CATALOG, catalog);
				attributeBean.setAttribute(Constants.LBL_VALUE, Constants.BLANK);
				attributesBean.setAttribute(attributeBean);					
			}
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RelevantDataProcessAndVersionAction::service():: ", sbe);
		}
		return attributesBean;
	}
}
