/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.bo;

import it.eng.console.hibernate.util.HibernateSessionFactory;
import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import javax.xml.registry.JAXRException;
import javax.xml.registry.infomodel.Service;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.spagic.metadb.base.TransitionStateConstants;
import org.spagic.metadb.base.TransitionStateUtils;
import org.spagic.metadb.model.Attribute;
import org.spagic.metadb.model.LogJournal;
import org.spagic.metadb.model.ProcessInstance;
import org.spagic.metadb.model.Step;
import org.spagic.metadb.model.TransitionInstance;
import org.spagic.registry.utils.IServiceRegistry;
import org.spagic.registry.utils.ServiceRegistryException;
import org.spagic.registry.utils.ServiceRegistryFactory;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

public class ProcessInstanceBO {
	public static Query getProcessInstanceList() throws HibernateException{
		return getProcessInstanceList(Constants.ALL_PROCESS, Constants.ALL_ITER, null);
	}
	
	public static Query getProcessInstanceList(long processId) throws HibernateException {
		return getProcessInstanceList(processId, Constants.ALL_ITER, null);
	}

	public static Query getProcessInstanceListByIterType(long poolType, List instancePool) throws HibernateException {
		return getProcessInstanceList(Constants.ALL_PROCESS, poolType, instancePool);
	}
	
	public static Query getProcessInstanceList(long processId, long idPoolType, List instancePool) throws HibernateException {
		return getProcessInstanceList(processId, null, idPoolType, null, null, null, null, null, null, instancePool, null);

	}
	
	public static Criteria getProcessInstanceList(String taxonomyName, String taxonomyValue) throws Exception{
		IServiceRegistry serviceRegistry = null;
		Criteria criteria = null;
		try {
			String serviceRegistryType = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.serviceRegistry");
	        serviceRegistry = ServiceRegistryFactory.getServiceRegistry(serviceRegistryType, null);
			serviceRegistry.initRegistry();
			List serviceList = serviceRegistry.getServicesClassifiedByTaxonomy(taxonomyName, taxonomyValue);
			int indexService;
			Service service;
			String uddiName, uddiOrganization;
			
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			criteria = session.createCriteria(ProcessInstance.class, "processInstance")
					.createCriteria("process", Criteria.INNER_JOIN);
			if(serviceList.size() > 0) {
				Disjunction orExpressions = Restrictions.disjunction();
				for(indexService=0; indexService<serviceList.size(); indexService++) {
					service = (Service)serviceList.get(indexService);
					uddiName = service.getName().getValue();				
					uddiOrganization = service.getProvidingOrganization().getName().getValue();
					orExpressions.add(Expression.and(Expression.eq("uddiOrg", uddiOrganization), Expression.eq("uddiName", uddiName)));
				}
				criteria.add(orExpressions);
			} else
				criteria.add(Expression.eq("process.idProcess", "-1"));
			
			criteria.addOrder(Order.asc("name"));
			criteria.addOrder(Order.desc("processInstance.startdate"));
			//join per recuperare iter e istanza di iter 
			criteria.createCriteria("processInstance.poolInstance", Criteria.LEFT_JOIN);
			
		} catch(ServiceRegistryException sre) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", sre);
			throw sre;
		} catch(JAXRException jax) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", jax);
			throw jax;
		} catch(Exception ex) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", ex);
			throw ex;
		} finally {
			serviceRegistry.finalizeRegistry();			
		}
		return criteria;
	}

	public static Query getProcessInstanceList(long process, String type, long idPoolType, String state, Date start, Date end, String org, String orderby, String orderType, List iterAttributes, List processAttributes) 
		throws HibernateException {			
		Query query = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			
			//*************  starting query ********************
			StringBuffer hqlBuffer = new StringBuffer();
			hqlBuffer.append("SELECT new list(processInstance.idProcessInstance, process.idProcess, process.name, ");
			if (idPoolType == Constants.NO_ITER)
				hqlBuffer.append(" '','',");
			else
				hqlBuffer.append(" poolType.name, "
						  + " ( SELECT istanceIterSub.uniquePoolInstanceKey FROM  org.spagic.metadb.model.PoolInstance istanceIterSub "
						  + "  WHERE istanceIterSub.idPool = processInstance.poolInstance.idPool ) as attributeValue, ");
		
			hqlBuffer.append(" process.uddiOrg, processInstance.startdate, processInstance.enddate, processInstance.state, processInstance.idCorrelation)"
							+ " FROM org.spagic.metadb.model.Process process," 
							+ " org.spagic.metadb.model.ProcessInstance processInstance ");
			
			//********* join sull'istanza di iter *******
			if (idPoolType == Constants.ALL_ITER) 
				hqlBuffer.append( " LEFT OUTER JOIN processInstance.poolInstance as poolInstance"
					+ " LEFT OUTER JOIN poolInstance.poolType as poolType");
			//iter selezionato
			else  if(idPoolType != Constants.NO_ITER)				
				hqlBuffer.append( " INNER JOIN processInstance.poolInstance as poolInstance"
							+ " INNER JOIN poolInstance.poolType as poolType"
							+ " WITH poolType.idPoolType = :idPoolType");
			//*********** fine join sull'istanza di iter ********
			
			hqlBuffer.append(" WHERE process.idProcess = processInstance.process.idProcess");
			
			if(type != null && !type.equalsIgnoreCase(Constants.ANY_TYPE))
				hqlBuffer.append(" AND process.typeService.value = :type");
			if(idPoolType == Constants.NO_ITER)
				hqlBuffer.append(" AND processInstance.poolInstance IS NULL");
			if(process != Constants.ALL_PROCESS)
				hqlBuffer.append(" AND process.idProcess = " + process);
			else 
				hqlBuffer.append(" AND process.processVersion = (SELECT max(processSubselect.processVersion)" 
					       + " FROM org.spagic.metadb.model.Process processSubselect"
					       + " WHERE processSubselect.name = process.name)");
			if(state!=null && !state.equals(""))
				hqlBuffer.append(" AND processInstance.state = " + state);
			if(start!=null) 					
				hqlBuffer.append(" AND processInstance.startdate >= :start_date");
			if(end!=null) 						
				hqlBuffer.append(" AND processInstance.enddate <= :end_date");
			if(org!=null && !org.equals(""))
				hqlBuffer.append(" AND process.uddiOrg = :org");
			
			
			//condizione relative agli attributi di istanza di iter 
			hqlBuffer.append(createInstancePoolConditions(iterAttributes));
			
			//condizioni relative agli atributi di processo (ovvero la somma di tutti quelli di transizione)
			hqlBuffer.append(createProcessRelevantDataConditions(processAttributes));
			
			//ricerca avanzata
			if(orderby!=null) {
				if(orderby.equalsIgnoreCase("DATA"))
					hqlBuffer.append(" ORDER BY processInstance.startdate " + orderType);
				else if(orderby.equalsIgnoreCase("OWNER"))
					hqlBuffer.append(" ORDER BY process.uddiOrg " + orderType + ", processInstance.startdate desc");
				else if(orderby.equalsIgnoreCase("PROCESS"))
					hqlBuffer.append(" ORDER BY process.name " + orderType + ", processInstance.startdate desc");
			}
			//ricerca senza aver specificato iter
			else if (idPoolType == Constants.ALL_ITER || idPoolType == Constants.NO_ITER)
				hqlBuffer.append(" ORDER BY processInstance.startdate DESC");
			else 
				hqlBuffer.append(" ORDER BY processInstance.poolInstance.idPool DESC, processInstance.startdate DESC");
			query = session.createQuery(hqlBuffer.toString());
			
			if(type != null && !type.equalsIgnoreCase(Constants.ANY_TYPE))
				query.setString("type", type);
			if(idPoolType != Constants.ALL_ITER && idPoolType != Constants.NO_ITER)
				query.setLong("idPoolType", idPoolType);
			if(start!=null)
				query.setTimestamp("start_date", start);
			if(end!=null) 	
				query.setTimestamp("end_date", end);
			if(org!=null && !org.equals(""))
				query.setString("org", org);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getProcessInstanceList:: ", he);
			throw he;
		}	
		return query;
	}

	private static String createProcessRelevantDataConditions(List<SourceBeanAttribute> processAttributes) {
		String hqlString = "";
		if(processAttributes != null && processAttributes.size() > 0 ) {
			Iterator<SourceBeanAttribute> it = processAttributes.iterator();
			SourceBeanAttribute attribute = null;
			long idAttribute;
			String valueAttribute;
			int prefixIndex = Constants.PREFIX_PROCESS_ATTRIBUTE.length();
			while(it.hasNext()) {
				attribute = it.next();
				idAttribute = (new Long(attribute.getKey().substring(prefixIndex))).longValue();
				valueAttribute = (String)attribute.getValue();
				hqlString += " AND EXISTS ( "
					+" FROM  org.spagic.metadb.model.RelevantData relevantDataSubselect,"
					+" org.spagic.metadb.model.RelevantDataProcessInstance relevantDataInstanceSubselect"
					+" WHERE relevantDataInstanceSubselect.relevantData.idRelevantData = relevantDataSubselect.idRelevantData"
					+" AND relevantDataSubselect.attribute.idAttribute = " + idAttribute 
					+" AND relevantDataSubselect.value like '%" + valueAttribute + "%'"
					+" AND relevantDataInstanceSubselect.processInstance.idProcessInstance = processInstance.idProcessInstance)";
			}
		}
		return hqlString;
	}
	
	private static String createInstancePoolConditions(List<SourceBeanAttribute> iterAttributes){
		StringBuffer hqlString = new StringBuffer();
		if(iterAttributes != null && iterAttributes.size() > 0 ) {
			Iterator<SourceBeanAttribute> it = iterAttributes.iterator();
			SourceBeanAttribute attribute = null;
			long idAttribute;
			String valueAttribute;
			int prefixIndex = Constants.PREFIX_ATTRIBUTE.length();
			while(it.hasNext()) {
				attribute = it.next();
				idAttribute = (new Long(attribute.getKey().substring(prefixIndex))).longValue();
				valueAttribute = (String)attribute.getValue();
				hqlString.append(" AND EXISTS ( "
						+" FROM  org.spagic.metadb.model.RelevantData relevantDataSubselect,"
						+" org.spagic.metadb.model.RelevantDataProcessInstance relevantDataInstance,"
						+" org.spagic.metadb.model.AttributePool attributePool"
						+" WHERE relevantDataSubselect.attribute.idAttribute = ");
				hqlString.append(idAttribute);
				hqlString.append(" AND relevantDataSubselect.value like '%");
				hqlString.append(valueAttribute);
				hqlString.append("%'"
						+" AND relevantDataSubselect.idRelevantData = relevantDataInstance.relevantData.idRelevantData"
						+" AND relevantDataInstance.processInstance.idProcessInstance = processInstance.idProcessInstance)");
			}
		}
		return hqlString.toString();
	}
			
	public static List getProcessInstanceIdList(long process, long idPoolType, String state, Date start, Date end, String org, String orderby, String orderType, List<SourceBeanAttribute> iterAttributes, List<SourceBeanAttribute> processAttributes) 
	throws HibernateException {			
	Query query = null;
	Session session = null;
	try {
		session = HibernateSessionFactory.getSessionFactory().getCurrentSession();

		//*************  starting query ********************
		StringBuffer hqlBuffer = new StringBuffer();
		hqlBuffer.append("SELECT processInstance.idProcessInstance" 
					+ " FROM Process process, ProcessInstance processInstance ");
		
		//********* join sull'istanza di iter *******
		if (idPoolType == Constants.ALL_ITER) 
			hqlBuffer.append(" LEFT OUTER JOIN processInstance.poolInstance as poolInstance"
				+ " LEFT OUTER JOIN poolInstance.poolType as poolType");
		//iter selezionato
		else  if(idPoolType != Constants.NO_ITER)				
			hqlBuffer.append(" INNER JOIN processInstance.poolInstance as poolInstance"
						+ " INNER JOIN poolInstance.poolType as poolType"
						+ " WITH poolType.idPoolType = :idPoolType");
		//*********** fine join sull'istanza di iter ********
		
						hqlBuffer.append(" WHERE process.idProcess = processInstance.process.idProcess");
		
		if(idPoolType == Constants.NO_ITER)
			hqlBuffer.append(" AND processInstance.poolInstance IS NULL");
		if(process != Constants.ALL_PROCESS)
			hqlBuffer.append(" AND process.idProcess = " + process);
		else 
			hqlBuffer.append(" AND process.processVersion = (SELECT max(processSubselect.processVersion)" 
				       + " FROM Process processSubselect"
				       + " WHERE processSubselect.name = process.name)");
		if(state!=null && !state.equals(""))
			hqlBuffer.append(" AND processInstance.state = " + state);
		if(start!=null) 					
			hqlBuffer.append(" AND processInstance.startdate >= :start_date");
		if(end!=null) 						
			hqlBuffer.append(" AND processInstance.enddate <= :end_date");
		if(org!=null && !org.equals(""))
			hqlBuffer.append(" AND process.uddiOrg = :org");
		
		
		//condizione relative agli attributi di istanza di iter 
		hqlBuffer.append(createInstancePoolConditions(iterAttributes));	
		
		//condizioni relative agli atributi di processo (ovvero la somma di tutti quelli di transizione)
		hqlBuffer.append(createProcessRelevantDataConditions(processAttributes));
		
		//ricerca avanzata
		if(orderby!=null) {
			if(orderby.equalsIgnoreCase("DATA"))
				hqlBuffer.append(" ORDER BY processInstance.startdate " + orderType);
			else if(orderby.equalsIgnoreCase("OWNER"))
				hqlBuffer.append(" ORDER BY process.uddiOrg " + orderType + ", processInstance.startdate desc");
			else if(orderby.equalsIgnoreCase("PROCESS"))
				hqlBuffer.append(" ORDER BY process.name " + orderType + ", processInstance.startdate desc");
		}
		//ricerca senza aver specificato iter
		else if (idPoolType == Constants.ALL_ITER || idPoolType == Constants.NO_ITER)
			hqlBuffer.append(" ORDER BY processInstance.startdate DESC");
		else 
			hqlBuffer.append(" ORDER BY processInstance.poolInstance.idPool DESC, processInstance.startdate DESC");
		query = session.createQuery(hqlBuffer.toString());
		if(idPoolType != Constants.ALL_ITER && idPoolType != Constants.NO_ITER)
			query.setLong("idPoolType", (new Long(idPoolType)).longValue());
		if(start!=null)
			query.setTimestamp("start_date", start);
		if(end!=null) 	
			query.setTimestamp("end_date", end);
		if(org!=null && !org.equals(""))
			query.setString("org", org);
	} catch(HibernateException he) {
		TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getProcessInstanceIdList:: ", he);
		throw he;
	}	
	return query.list();
}

	public static Query getMessagesList(long processId, long idStep, String state, Date start, Date end, String orderType, List processAttributes) 
	throws HibernateException {			
		Query query = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			
			//*************  starting query ********************
			StringBuffer hqlBuffer = new StringBuffer();
			hqlBuffer.append("SELECT new list(transitionInstance.idJournal,"
							+ " transitionInstance.processInstance.idProcessInstance,"
							+ " transitionInstance.transitionState.idState,"
							+ " transitionInstance.transitionStart,"
							+ " transitionInstance.transitionEnd)" 
							+ " FROM TransitionInstance transitionInstance," 
							+ " ProcessInstance processInstance"
							+ " WHERE transitionInstance.processInstance.idProcessInstance = processInstance.idProcessInstance");
			if(idStep != Constants.ANY_TYPE_COMPONENT )
				hqlBuffer.append(" AND transitionInstance.stepByIdStepTarget.idStep = :idStep");
			else 
				hqlBuffer.append(" AND processInstance.process.idProcess = :idProcess");
			if(state!=null && !state.equals("")) 
				hqlBuffer.append(" AND processInstance.state = " + state);
			if(start!=null) 					
				hqlBuffer.append(" AND transitionInstance.transitionStart >= :start_date");
			if(end!=null) 						
				hqlBuffer.append(" AND transitionInstance.transitionStart <= :end_date");
			
			//condizioni relative agli atributi di processo (ovvero la somma di tutti quelli di transizione)
			hqlBuffer.append(createProcessRelevantDataConditions(processAttributes));
			
			hqlBuffer.append(" ORDER BY transitionInstance.transitionStart " + orderType);
			query = session.createQuery(hqlBuffer.toString());
			if(idStep != Constants.ANY_TYPE_COMPONENT )
				query.setLong("idStep", idStep);
			else 
				query.setLong("idProcess", processId);
			if(start!=null)
				query.setTimestamp("start_date", start);
			if(end!=null) 	
				query.setTimestamp("end_date", end);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getMessagesList: ", he);
			throw he;
		}	
		return query;
	}

	public static List getMessagesIdList(long idStep, String state, Date start, Date end, String orderType, List processAttributes) 
		throws HibernateException {			
		Query query = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			
			//*************  starting query ********************
			StringBuffer hqlBuffer = new StringBuffer();
			hqlBuffer.append("SELECT transitionInstance.idJournal"
							+ " FROM TransitionInstance transitionInstance, ProcessInstance processInstance"
							+ " WHERE transitionInstance.stepByIdStepTarget.idStep = :idStep"
							+ " AND transitionInstance.processInstance.idProcessInstance = processInstance.idProcessInstance");

			if(state!=null && !state.equals("")) 
				hqlBuffer.append(" AND processInstance.state = " + state);
			if(start!=null) 					
				hqlBuffer.append(" AND transitionInstance.transitionStart >= :start_date");
			if(end!=null) 						
				hqlBuffer.append(" AND transitionInstance.transitionStart <= :end_date");
			
			//condizioni relative agli atributi di processo (ovvero la somma di tutti quelli di transizione)
			hqlBuffer.append(createProcessRelevantDataConditions(processAttributes));
			
			hqlBuffer.append(" ORDER BY transitionInstance.transitionStart " + orderType);
			query = session.createQuery(hqlBuffer.toString());
			query.setLong("idStep", idStep);
			if(start!=null)
				query.setTimestamp("start_date", start);
			if(end!=null) 	
				query.setTimestamp("end_date", end);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getMessagesIdList: ", he);
			throw he;
		}	
		return query.list();
	}

	public static List<TransitionInstance> getTransitionInstance(long stepId, long processInstanceId) 
	throws HibernateException {
		List<TransitionInstance> transitionInstance = null;
		Monitor monitor = null;
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			monitor = MonitorFactory.start("hibernate.accesstransitionInstance");
			Criteria criteria = session.createCriteria(TransitionInstance.class);
			criteria.add(Expression.eq("stepByIdStepTarget.idStep", stepId));
			criteria.add(Expression.eq("processInstance.idProcessInstance", processInstanceId));
			criteria.addOrder(Order.asc("transitionStart"));
			transitionInstance = criteria.list();
			monitor.stop();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getTransitionInstance(long stepId, long processInstanceId):: ", he);
			monitor.stop();
			throw he;
		} 	
		return transitionInstance;
	}
	
	public static TransitionInstance getTransitionInstance(long idJournal) 
	throws HibernateException {
		TransitionInstance transitionInstance = null;
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Criteria criteria = session.createCriteria(TransitionInstance.class);
			criteria.add(Expression.eq("idJournal", idJournal));
			criteria.addOrder(Order.asc("transitionStart"));
			transitionInstance = (TransitionInstance)criteria.uniqueResult();			
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getTransitionInstance(long idJournal):: ", he);
			throw he;
		} 	
		return transitionInstance;
	}
	
	public static List getAttachmentsNumber(long idJournal) 
	throws HibernateException {			
	Query query = null;
	Session session = null;
	try {
		session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
		StringBuffer hqlBuffer = new StringBuffer();
		hqlBuffer.append("SELECT new list(transitionInstance.attachmentIn, transitionInstance.attachmentOut) "
						+ " FROM TransitionInstance transitionInstance"
						+ " WHERE transitionInstance.idJournal = :idJournal");
		query = session.createQuery(hqlBuffer.toString());
		query.setLong("idJournal", idJournal);
	} catch(HibernateException he) {
		TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getAttachmentNumber: ", he);
		throw he;
	}	
	return query.list();
}
	
	public static boolean isTransitionDataNotNull(long idJournal) 
	throws HibernateException {			
	Query query = null;
	Session session = null;
	try {
		session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
		StringBuffer hqlBuffer = new StringBuffer();
		hqlBuffer.append("SELECT transitionInstance.idJournal "
						+ " FROM TransitionInstance transitionInstance"
						+ " WHERE transitionInstance.idJournal = :idJournal " 
						+ " AND transitionInstance.transitionData is null "
						+ " AND transitionInstance.transitionDataOut is null ");
		query = session.createQuery(hqlBuffer.toString());
		query.setLong("idJournal", idJournal);
	} catch(HibernateException he) {
		TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::isTransitionDataNull: ", he);
		throw he;
	}	
	if (query.list().size() > 0)
		return false;
	return true;
	}
	
	public static List getLogsJournal(long idJournal) throws HibernateException {
		List logsJournal = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Criteria criteria = session.createCriteria(LogJournal.class);
			criteria.add(Expression.eq("transitionInstance.idJournal", idJournal));
			criteria.addOrder(Order.asc("datelog"));
			logsJournal = criteria.list();	
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getLogsJournal(long idJournal): ", he);
			throw he;
		} 		
		return logsJournal;
	}
	
	public static int getExecutionStateInstanceService(Step aStep, long processInstanceId) 
		throws HibernateException {
		if (aStep.isIsStart())
			return TransitionStateConstants.TRANSITION_DONE;
		List<TransitionInstance> transitionInstanceList = getTransitionInstance(aStep.getIdStep(), processInstanceId);
		for (TransitionInstance transitionInstance : transitionInstanceList) {
			int state = transitionInstance.getTransitionState().getIdState();
			if (TransitionStateUtils.isFaultState(state))
				return TransitionStateConstants.TRANSITION_FAULTED;
			if (TransitionStateUtils.isActiveState(state))
				return TransitionStateConstants.TRANSITION_ACTIVE;
			if (TransitionStateUtils.isDoneState(state))			
				return TransitionStateConstants.TRANSITION_DONE;
		}
		return TransitionStateConstants.TRANSITION_MISSING;
	}
		
	
	
	public static long getCountProcessInstances(long processId){
		long processInstance = 0;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			String hqlString = "SELECT count(processInstance.idProcessInstance) " 	 
							+ " FROM ProcessInstance processInstance "								
							+ " WHERE processInstance.process.idProcess = :idProcess";
			
			Query query = session.createQuery(hqlString);
			query.setLong("idProcess", processId);
			
			processInstance = (Long)query.list().get(0);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getInstanceSa:: ", he);
			throw he;
		} 		
		return processInstance;
	}

	
	public static long getCountProcessInstancesByState(long processId, long idState, Date fromDate){
		long processInstance = 0;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			String hqlString = "SELECT count(processInstance.idProcessInstance) "
							+ " FROM ProcessInstance processInstance "
							+ " WHERE processInstance.process.idProcess = :idProcess AND"
							+ " processInstance.state = :idState";
			if (fromDate != null)
				hqlString += " AND processInstance.enddate > :pFromDate ";
			Query query = session.createQuery(hqlString);
			query.setLong("idProcess", processId);
			query.setLong("idState", idState);
			if (fromDate != null)
				query.setTimestamp("pFromDate", fromDate);
			processInstance = (Long)query.list().get(0);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getInstanceSa:: ", he);
			throw he;
		} 		
		return processInstance;
	}
	
	public static double getMeanExecutionTime(long processId, long idState, Date fromDate){
		
		Session session = null;
		try {
			
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			String hqlString = "SELECT avg(processInstance.executionTime) "
							+ " FROM ProcessInstance processInstance "
							+ " WHERE processInstance.process.idProcess = :idProcess AND"
							+ " processInstance.state = :idState";
			if (fromDate != null)
				hqlString += " AND processInstance.enddate > :pFromDate ";
			Query query = session.createQuery(hqlString);
			query.setLong("idProcess", processId);
			query.setLong("idState", idState);
			if (fromDate != null)
				query.setTimestamp("pFromDate", fromDate);
			Object o = query.uniqueResult();
			if (o != null){
				return (Double)query.uniqueResult() / 1000;
			}else{
				return 0 * 1000;
			}
		
			
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getInstanceSa:: ", he);
			throw he;
		} 		

	}
	
	
	public static long getCountProcessInstances(long processId, long idPoolType){
		long processInstance = 0;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			StringBuffer hqlBuffer = new StringBuffer();
			hqlBuffer.append("SELECT count(processInstance.idProcessInstance) " 	 
							+ " FROM org.spagic.metadb.model.ProcessInstance processInstance");	
			hqlBuffer.append(" WHERE processInstance.process.idProcess = :idProcess");
			//iter selezionato
			if (idPoolType != Constants.ALL_ITER && idPoolType != Constants.NO_ITER) 				
				hqlBuffer.append(" AND processInstance.poolInstance.poolType.idPoolType = :idPoolType");
			if (idPoolType == Constants.NO_ITER)
				hqlBuffer.append(" AND processInstance.poolInstance IS NULL");
			
			Query query = session.createQuery(hqlBuffer.toString());
			query.setLong("idProcess", processId);
			if (idPoolType != Constants.ALL_ITER && idPoolType != Constants.NO_ITER)
				query.setLong("idPoolType", idPoolType);
			
			processInstance = (Long)query.list().get(0);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getCountInstancesSa:: ", he);
			throw he;
		} 		
		return processInstance;
	}
	
	public static String getAttributeName(long idAttribute) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Attribute attribute = (Attribute)session.get(Attribute.class, idAttribute);	
			return attribute.getName();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getAttributeName: ", he);
			throw he;
		}
	}
	
	public static int getStateInstanceProcess(long instanceId) {
		Session session = null;
		try {
			TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION, "ProcessInstanceBO::Getting Process Instance with ID --> ["+instanceId+"]");
			String hqlQuery = "from org.spagic.metadb.model.ProcessInstance pi where pi.idProcessInstance = :p1";
			TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION,"ProcessInstanceBO::getStateInstanceProcess -> Using Current Session ");
			session = HibernateSessionFactory.getSessionFactory().openSession();
			Query q = session.createQuery(hqlQuery);
			TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION,"ProcessInstanceBO::getStateInstanceProcess -> Using Current Session ");
			q.setLong("p1", instanceId);
			
			ProcessInstance pi = (ProcessInstance)q.uniqueResult();
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL,(pi != null ? " Process Instance is Not Null" : " Process Instance is NULL"));
			return pi.getState();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getStateInstanceProcess: ", he);
			throw he;
		}finally{
			session.close();
		}
	}

	public static int getStateJournal(long idJournal) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			TransitionInstance journal = (TransitionInstance)session.get(TransitionInstance.class, idJournal);	
			return journal.getTransitionState().getIdState();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getStateJournal: ", he);
			throw he;
		}
	}
	
	public static long getIdJournal(long idLog) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			LogJournal log = (LogJournal)session.get(LogJournal.class, idLog);	
			return log.getTransitionInstance().getIdJournal();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getIdJournal: ", he);
			throw he;
		}
	}
	
	public static List getInstanceSummaryInfo(long idInstance) {
		Query query = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			//*************  starting query ********************
			String hqlString = "SELECT new list(process.name, processInstance.state, process.idProcess, processInstance.idCorrelation)"
							+ " FROM ProcessInstance processInstance, Process process "
							+ " WHERE processInstance.idProcessInstance = :idInstance AND processInstance.process = process";
			
			query = session.createQuery(hqlString);
			query.setLong("idInstance", idInstance);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getInstanceSummaryInfo: ", he);
			throw he;
		}	
		List summaryInfo = query.list();
		return (List)summaryInfo.get(0);				
	}
	
	public static Long[] getNextLinkedProcess(long processInstanceId){
		Query query = null;
		Session session = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			//*************  starting query ********************
			String hqlString = "SELECT new list(processInstance.idProcessInstance, processInstance.process.idProcess)"
							+ " FROM ProcessInstance processInstance"
							+ " WHERE processInstance.poolInstance.idPool = "
							+ " (SELECT processInstance.poolInstance.idPool FROM ProcessInstance processInstance WHERE processInstance.idProcessInstance= :idInstance)"
							+ " ORDER BY processInstance.startdate ASC";
			query = session.createQuery(hqlString);
			query.setLong("idInstance", processInstanceId);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getNextAssociatedProcess: ", he);
			throw he;
		}	
		List<List> processInstancesList = query.list();
		Iterator<List> it = processInstancesList.iterator();
		long instanceId;
		Long[] result = null; 
		while(it.hasNext()) {
			instanceId = (Long)it.next().get(0);
			if (instanceId == processInstanceId) 
				try {
					List<Long> infoInstanceLinked = it.next();
					result = new Long[2];
					result[0] = infoInstanceLinked.get(0);
					result[1] = infoInstanceLinked.get(1);
					return result;
				}catch(NoSuchElementException nse){
					//is last element
					break;
				}
		}
		return result;	
	}
	
	
}
