/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.parser;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Iterator;
import java.util.List;

import org.spagic.metadb.base.TransitionStateConstants;
import org.spagic.metadb.model.Step;

public class SvgParser {
	long _processId;
	long _processInstanceId;
	//_svgBean can contains processSvg or processInstanceSvg
	SourceBean _svgBean = null;	
	public static final String NODE_LABEL = "node";
	public static final String EDGE_LABEL = "edge";
	
	public SvgParser(long processId) throws SourceBeanException {
		_processId = processId;
		initInputSvgBean();					
	}

	public SvgParser(long processId, long processInstanceId) throws SourceBeanException {
		_processId = processId;
		_processInstanceId = processInstanceId;
		initInputSvgBean();
	}
	
	private void initInputSvgBean() throws SourceBeanException {
		try {
			String roughSvg = ProcessBO.getRoughSvg(_processId);
			_svgBean = SourceBean.fromXMLString(roughSvg, true, false);			
		} catch(SourceBeanException sbe) {		
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "SvgParser::initInputSvgBean: ", sbe);
			throw sbe;
		}		
	}

	public String createProcessSvgString() throws SourceBeanException {
		try {			
			String width = (String)_svgBean.getAttribute("width");
			_svgBean.updAttribute("width", getBiggerWidth(width));
			List<SourceBean> nodeList = _svgBean.getFilteredSourceBeanAttributeAsList("g.g", "class", NODE_LABEL);
			String stepName = null;		
			Step step = null;			
			for (SourceBean nodeBean : nodeList) {
				nodeBean.updAttribute("a.href", "javascript:void(0)");
				stepName = nodeBean.getCharacters("title");
				// Compatibility check
				if (stepName == null) {
					stepName = nodeBean.getCharacters("a.text");
				}
				
				modifyCoordinateNextText((String)nodeBean.getAttribute("a.text.y"));
				nodeBean.updAttribute("id", stepName);
				nodeBean.setAttribute("onclick", "if (nodeProcessLink == 'FALSE') showDetail(evt); else nodeProcessLink = 'FALSE';");
				
				// Remove process name from step name
				step = ProcessBO.getStepDetail(stepName);
				String decoratedStepName = decorateStepName(step.getName());
				
				int dotPos = decoratedStepName.indexOf('.');
				if (dotPos != -1) {
					decoratedStepName = decoratedStepName.substring(dotPos + 1);
				}
				nodeBean.setCharacters("a.text", decoratedStepName);
				nodeBean.updAttribute("a.text.style", "font-size:6.00pt;");				
			}			
			_svgBean.setAttribute(getCDATABean());			
		} catch(SourceBeanException sbe) {		
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "SvgParser::createProcessSvgString: ", sbe);
			throw sbe;
		}
		return _svgBean.toString();
	}
	
	private String decorateStepName(String stepName) {
		// Remove process name from step name
		int dotPos = stepName.indexOf('.');
		if (dotPos != -1) {
			stepName = stepName.substring(dotPos + 1);
		}
		
		// Remove also the process version
		int versionPos = stepName.indexOf("_v_");
		if (versionPos != -1) {
			stepName = stepName.substring(0, versionPos);
		}
		
		return stepName;
	}
	
	private void modifyCoordinateNextText(String y) throws SourceBeanException {			
		List textBeanList = _svgBean.getFilteredSourceBeanAttributeAsList("g.g", "a.text.y", y);		
		Iterator it = textBeanList.iterator();
		SourceBean textBean = null;
		try {
			if(textBeanList.size()>1) {
				double startY = Double.parseDouble(y) - 6;
				double distance = 11;
				while(it.hasNext()) {
					textBean = (SourceBean)it.next();
					textBean.updAttribute("a.text.y", Double.toString(startY));
					startY += distance; 					
				}
			}
		} catch(SourceBeanException sbe) {		
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "SvgParser::modifyCoordinateNextText: ", sbe);
			throw sbe;
		}
	}
	
	private SourceBean getCDATABean() throws SourceBeanException{
		SourceBean scriptBean = null;
		try {
			scriptBean = new SourceBean("script", true, false);
			scriptBean.setAttribute("type", "text/javascript");
			String var = "var nodeProcessLink = 'FALSE';\n";
			String functionshowDetailString = "function showDetail(evt){\n"
												+ "	var elem = evt.target.parentNode.parentNode;\n"
								  				+ "	var stepId = elem.getAttribute('id');\n"
								  				+ "	top.showDetailParent(stepId);\n"
												+"}";
			scriptBean.setCharacters(var + " " + functionshowDetailString);
		} catch(SourceBeanException sbe) {		
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "SvgParser::getCDATABean: ", sbe);
			throw sbe;
		}
		return scriptBean;
	}
	
	private String getBiggerWidth(String width) {
		if(width.indexOf("pt") != -1) {
			String widthTemp = width.substring(0, width.indexOf("pt")); 
			int newWidth = Integer.parseInt(widthTemp) + 70; 
			return Integer.toString(newWidth) + "pt";
		} else {
			String widthTemp = width.substring(0, width.indexOf("in")); 
			double newWidth = Double.parseDouble(widthTemp) + 2; 
			return Double.toString(newWidth) + "in";
		}
	}

	public String createProcessInstanceSvgString() throws SourceBeanException {
		createProcessSvgString();
		Long[] processInstanceIdLinked = ProcessInstanceBO.getNextLinkedProcess(_processInstanceId);
		List<SourceBean> nodeList = _svgBean.getFilteredSourceBeanAttributeAsList("g.g", "class", NODE_LABEL);
		String stepName;
		int stateInstanceService;
		for (SourceBean nodeBean : nodeList) {
			stepName = nodeBean.getCharacters("title");
			// Compatibility check for old graphs
			if (stepName == null) {
				stepName = nodeBean.getCharacters("a.text");
			}
			
			// Retrieve process state			
			Step step = ProcessBO.getStepDetail(stepName);
			stateInstanceService = ProcessInstanceBO.getExecutionStateInstanceService(step, _processInstanceId);
			changeNodeState(stateInstanceService, nodeBean);
			//se esite un instanza appartenete alla stessa istanza di iter
			//viene linkata sui nodi finali
			if(processInstanceIdLinked != null 
					&& step.isIsEnd() 
					&& isExecuted(stateInstanceService)) {
				SourceBean aNode = (SourceBean)nodeBean.getAttribute("a");
				SourceBean textNode = (SourceBean)aNode.getAttribute("text");
				textNode.updAttribute("onclick", "nodeProcessLink='TRUE';top.showLinkedInstance(" + processInstanceIdLinked[0] + "," + processInstanceIdLinked[1] + ")");
				textNode.setCharacters(ProcessBO.getProcessDetail(processInstanceIdLinked[1]).getName());
				changeColor("fill:", "red", textNode);
				changeColor("font-weight:", "bold", textNode);
				
				//create a new href containing textNode
				SourceBean hrefNode = new SourceBean("a");
				hrefNode.setAttribute("href", "#");
				hrefNode.setAttribute("title", "Show Process");
				hrefNode.setAttribute(textNode);
				
				aNode.delAttribute("text");
				aNode.setAttribute(hrefNode);
			}
		}		
		return _svgBean.toString();
	}

	private boolean isExecuted(int state) {
		boolean isExecuted = false;
		switch (state) {
		case TransitionStateConstants.TRANSITION_DONE:
			return true;
		case TransitionStateConstants.TRANSITION_ACTIVE:
			return true;
		case TransitionStateConstants.TRANSITION_FAULTED:
			return true;
		}
		return isExecuted;
	}
	
	private void changeNodeState(int state, SourceBean node) throws SourceBeanException{
		String newColor = null;
		switch (state) {
		case TransitionStateConstants.TRANSITION_DONE:
			// Green
			newColor = "#99FF99";// "#33FF99"
			break;
		case TransitionStateConstants.TRANSITION_ACTIVE:
			// Yellow
			newColor = "#FFFF99";
			break;
		case TransitionStateConstants.TRANSITION_FAULTED:
			newColor = "red";
			break;
		}
		
		if (newColor == null) {
			return;
		}
		
		SourceBean shape = (SourceBean)node.getAttribute("a.polygon");
		if (shape != null) {
			List<SourceBean> nodeList = node.getAttributeAsList("a.path");
			
			if (nodeList.isEmpty()) {
				// Mantain border color
				changeFillColor(newColor, shape);
			} else {
				// Color also the border
				changeFillColor(newColor, shape);
				changeStrokeColor(newColor, shape);

				for (SourceBean polyLine : nodeList)
					changeFillColor(newColor, polyLine);
			}
		} else {
			shape = (SourceBean)node.getAttribute("a.ellipse");
			if (shape != null) {
				changeFillColor(newColor, shape);
			}
		}
	}

	private void changeFillColor(String newColor,
			SourceBean node) throws SourceBeanException {
		changeColor("fill:", newColor, node);
	}
	
	private void changeStrokeColor(String newColor,
			SourceBean node) throws SourceBeanException {
		changeColor("stroke:", newColor, node);
	}

	private void changeColor(String attribute, String newColor,
			SourceBean node) throws SourceBeanException {
		String style = (String) node.getAttribute("style");
		
		int attributePos = style.indexOf(attribute);
		if (attributePos != -1) {
			int attributeEndPos = style.indexOf(';', attributePos);
			style = style.substring(0, attributePos) + attribute + newColor + style.substring(attributeEndPos);
		}
		else {
			style = style + attribute + newColor + ";";
		}
		
		try {
			node.updAttribute("style", style);
		} catch (SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL,
					"SvgParser::changeColor: ", sbe);
			throw sbe;
		}
	}
	
}
