/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.process.bo;

import it.eng.console.hibernate.util.HibernateSessionFactory;
import it.eng.console.util.Constants;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Expression;
import org.spagic.metadb.model.AttributePool;

public class RelevantDataBO {
	
	public static List getAllRelevantData(long processInstanceId) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			String hqlString = "SELECT DISTINCT new list(a.idAttribute, a.name, rd.value, c.name) "
			+" FROM Attribute a, RelevantData rd, RelevantDataProcessInstance rdi"
			+" LEFT JOIN a.attributeCatalogs ac "
			+" LEFT JOIN ac.catalog c"
			+" WHERE a=rd.attribute"
			+" AND rdi IN elements(rd.relevantDataProcessInstances)"
			+" AND rdi.processInstance.idProcessInstance=" + processInstanceId
			+" ORDER BY c.name ASC";
			Query query = session.createQuery(hqlString);
			List relevantData = query.list();
			
			return relevantData;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RelevantDataBO::getAllRelevantData: ", he);
			throw he;
		}
	}	
	
	public static List<List> getAttributeRelevant(long processId) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			
			String hqlString = "SELECT new list(a.idAttribute, a.name, c.name) "
				+" FROM Attribute a, Process sa, Rule rule, StepRule stepRule"
				+" LEFT JOIN a.attributeCatalogs ac "
				+" LEFT JOIN ac.catalog c"
				+" WHERE a=rule.attribute"
				+" AND rule=stepRule.rule"
				+" AND stepRule.step IN elements(sa.steps)"
				+" AND sa.idProcess=" + processId
				+" ORDER BY c.name ASC";
			Query query = session.createQuery(hqlString);
			List relevantData = query.list();
			return relevantData;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RelevantDataBO::getAllRelevantData: ", he);
			throw he;
		}
	}	
	
	public static List getRelevantData(long idJournal) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			String hqlString = "SELECT new list(a.name, rd.value, c.name) "
				+" FROM Attribute a, RelevantData rd"
				+" LEFT JOIN a.attributeCatalogs as attributeCatalog"
				+" LEFT JOIN attributeCatalog.catalog c"
				+" WHERE rd.logJournal.transitionInstance.idJournal = " + idJournal
				+" AND rd.attribute.idAttribute = a.idAttribute"
				+" ORDER BY c.name ASC";
			Query query = session.createQuery(hqlString);
			List relevantData = query.list();
			
			return relevantData;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RelevantDataBO::getProcessRelevantData: ", he);
			throw he;
		}
	}	
	
	public static List getAttributes(long idPoolType) {
		Session session = null;
		List attributesList = null;
		try {
			session = HibernateSessionFactory.getSessionFactory().openSession();
			Criteria criteria = session.createCriteria(AttributePool.class);
			criteria.add(Expression.eq("poolType.idPoolType", idPoolType));
			//criteria.addOrder(Order.asc("attribute.name"));
			attributesList = criteria.list();
		} catch (HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RelevantDataBO::getAttributes: ", he);
			throw he;
		} 
		return attributesList;
	}
}
