package it.eng.console.security.action;

import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.security.IAuthenticationHandler;
import it.eng.spago.security.IAuthorizationHandler;
import it.eng.spago.tracing.TracerSingleton;

//LoginAction is similar to GenericLoginHandler (Spago)
public class LoginAction extends AbstractAction {

    public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws EMFInternalError {
        if(serviceRequest.getAttribute("START") == null){
	    	IAuthenticationHandler authenticationHandler = getAuthenticationHadler();
	        IAuthorizationHandler authorizationHandler = getAuthorizationHandler();
	    	Object userPrincipal = authenticationHandler.authenticate(serviceRequest, serviceResponse);
	    	authorizationHandler.putAuhtorizationProfile(userPrincipal, getRequestContainer());
        }
    }
        

    private IAuthenticationHandler getAuthenticationHadler() throws EMFInternalError {
        SourceBean authenticationHandler = (SourceBean)
            ConfigSingleton.getInstance().getAttribute("SECURITY.AUTHENTICATION_HANDLER");
        String authenticationHandlerClassName = (String)authenticationHandler.getCharacters();
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG,
            "LoginAction::getAuthenticationHadler: authenticationHandlerClassName [ " +
            authenticationHandlerClassName + " ] ");
        try {
            return (IAuthenticationHandler)Class.forName(authenticationHandlerClassName).newInstance();
        } // try
        catch (InstantiationException ie) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, ie);
        } // catch (InstantiationException ie)
        catch (IllegalAccessException iae) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, iae);
        } // catch (InstantiationException iae)
        catch (ClassNotFoundException cnfe) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, cnfe);
        } // catch (InstantiationException cnfe)
    } 

    private IAuthorizationHandler getAuthorizationHandler() throws EMFInternalError {
        SourceBean authorizationHandler = (SourceBean)
            ConfigSingleton.getInstance().getAttribute("SECURITY.AUTHORIZATION_HANDLER");
        String authorizationHandlerClassName = (String)authorizationHandler.getCharacters();
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG,
            "LoginAction::getAuthenticationHadler: authorizationHandlerClassName [ " +
            authorizationHandlerClassName + " ] ");
        try {
            return (IAuthorizationHandler)Class.forName(authorizationHandlerClassName).newInstance();
        } // try
        catch (InstantiationException ie) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, ie);
        } // catch (InstantiationException ie)
        catch (IllegalAccessException iae) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, iae);
        } // catch (InstantiationException iae)
        catch (ClassNotFoundException cnfe) {
            throw new EMFInternalError(EMFErrorSeverity.ERROR, cnfe);
        } // catch (InstantiationException cnfe)
    } 
}
