package it.eng.spago.base;

import it.eng.spago.tracing.TracerSingleton;

import java.io.StringWriter;

import org.apache.commons.betwixt.IntrospectionConfiguration;
import org.apache.commons.betwixt.io.BeanWriter;
import org.apache.commons.betwixt.strategy.DecapitalizeNameMapper;
import org.apache.commons.betwixt.strategy.HyphenatedNameMapper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * XML serializer with the Betwixt library
 */
public class BetwixtXMLSerializer implements XMLSerializer {

	public void serialize(Document document, Element element, final String key, final Object source) {
		StringWriter sw = new StringWriter();
		BeanWriter writer = new BeanWriter(sw);
		IntrospectionConfiguration config = writer.getXMLIntrospector().getConfiguration();
		
		config.setAttributesForPrimitives(true);
		writer.enablePrettyPrint();
		writer.getBindingConfiguration().setMapIDs(false);

		// set a custom name mapper for attributes
		config.setAttributeNameMapper(new HyphenatedNameMapper());
		// set a custom name mapper for elements
		config.setElementNameMapper(new DecapitalizeNameMapper());
		
		writer.getXMLIntrospector().setConfiguration(config);

		// write out the bean
		try {
			writer.write(source);
		} catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "BetwixtXMLSerializer::serialize: ", ex);
		}

		try {
			// Object->XML String->SourceBean->Element
			// Unfortunately, after all conversions the DOM tree is converted to String
			SourceBean sbFromXML = SourceBean.fromXMLString(sw.getBuffer().toString());
			// Create element that contains converted object
			Element keyElement = document.createElement(key.toUpperCase());
			
			// Append converted objects to the new created element
			keyElement.appendChild(sbFromXML.toElement(document));
			element.appendChild(keyElement);
		} catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "BetwixtXMLSerializer::serialize: ", ex);
		}
	}

}
